
from flask import Flask
from active_sqlalchemy import SQLAlchemy

app = Flask(__name__)
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite://'

db = SQLAlchemy()

class User(db.Model):
    name = db.Column(db.String(25))
    location = db.Column(db.String(50), default="USA")
    last_access = db.Column(db.DateTime)

db.create_all()



@app.route("/")
def index():
    u = User.create(name="Jones")
    return "Hello %s" % u.name



if __name__ == "__main__":

    app.run(debug=True)


"""
Some common functions
"""
from __future__ import division
import os
import re
import string
import random
import urlparse
import socket
import subprocess
import functools
import multiprocessing
import threading
from passlib.hash import bcrypt as crypt_engine
import slugify
import jinja2


def get_base_dir():
    """
    Return the base directory
    """
    return os.path.split(os.path.abspath(os.path.dirname(__file__)))[0]


def is_valid_email(email):
    """
    Check if email is valid
    """
    pattern = re.compile(r'[\w\.-]+@[\w\.-]+[.]\w+')
    return True if pattern.match(email) else False


def is_valid_password(password):
    """
    Check if a password is valid
    """
    pattern = re.compile(r"^.{4,25}$")
    return True if pattern.match(password) else False


def is_valid_url(url):
    """
    Check if url is valid
    """
    regex = re.compile(
            r'^(?:http|ftp)s?://' # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
            #r'localhost|' #localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
            r'(?::\d+)?' # optional port
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)
    return bool(regex.match(url))


def to_struct(**kwargs):
    """
    Convert kwargs to struct
     a = kwargs_to_object(Name='Hello', Value='World' )
     a.Name -> Hello

     :return object:
    """
    return type('', (), kwargs)

def get_domain_name(url):
    """
    Get the domain name
    :param url:
    :return:
    """
    if not url.startswith("http"):
        url = "http://" + url
    if not is_valid_url(url):
        raise ValueError("Invalid URL '%s'" % url)
    parse = urlparse.urlparse(url)
    return parse.netloc


def seconds_to_time(sec):
    """
    Convert seconds into time H:M:S
    """
    return "%02d:%02d" % divmod(sec, 60)


def time_to_seconds(t):
    """
    Convert time H:M:S to seconds
    """
    l = list(map(int, t.split(':')))
    return sum(n * sec for n, sec in zip(l[::-1], (1, 60, 3600)))

def slug(string):
    """
    Create a string to slug
    :param string:
    :return:
    """
    return slugify.slugify(string)


def encrypt_string(string):
    """
    Encrypt a string
    """
    return crypt_engine.encrypt(string)


def verify_encrypted_string(string, encrypted_string):
    """
    Verify an encrypted string
    """
    return crypt_engine.verify(string, encrypted_string)

def generate_random_string(length=8):
    """
    Generate a random string
    """
    char_set = string.ascii_uppercase + string.digits
    return ''.join(random.sample(char_set * (length - 1), length))

def generate_random_hash(size=32):
    """
    Return a random hash key
    :param size: The max size of the hash
    :return: string
    """
    return os.urandom(size//2).encode('hex')

def format_number(value):
    """
    Format a number returns it with comma separated
    """
    return "{:,}".format(value)


def filter_stopwords(str):
    """
    Stop word filter
    returns list
    """
    STOPWORDS = ['a', 'able', 'about', 'across', 'after', 'all', 'almost',
                 'also', 'am', 'among', 'an', 'and', 'any', 'are', 'as', 'at',
                 'be', 'because', 'been', 'but', 'by', 'can', 'cannot',
                 'could', 'dear', 'did', 'do', 'does', 'either', 'else',
                 'ever', 'every', 'for', 'from', 'get', 'got', 'had', 'has',
                 'have', 'he', 'her', 'hers', 'him', 'his', 'how', 'however',
                 'i', 'if', 'in', 'into', 'is', 'it', 'its', 'just', 'least',
                 'let', 'like', 'likely', 'may', 'me', 'might', 'most', 'must',
                 'my', 'neither', 'no', 'nor', 'not', 'of', 'off', 'often',
                 'on', 'only', 'or', 'other', 'our', 'own', 'rather', 'said',
                 'say', 'says', 'she', 'should', 'since', 'so', 'some', 'than',
                 'that', 'the', 'their', 'them', 'then', 'there', 'these',
                 'they', 'this', 'tis', 'to', 'too', 'twas', 'us',
                 'wants', 'was', 'we', 'were', 'what', 'when', 'where',
                 'which', 'while', 'who', 'whom', 'why', 'will', 'with',
                 'would', 'yet', 'you', 'your']

    return [t for t in str.split() if t.lower() not in STOPWORDS]


def to_currency(amount, add_decimal=True):
    """
    Return the US currency format
    """
    return '{:1,.2f}'.format(amount) if add_decimal else '{:1,}'.format(amount)

def is_port_open(port, host="127.0.0.1"):
    """
    Check if a port is open
    :param port:
    :param host:
    :return bool:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((host, int(port)))
        s.shutdown(2)
        return True
    except Exception as e:
        return False

def run(cmd):
    process = subprocess.Popen(cmd, shell=True,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE)
    return process.communicate()[0].strip()


def convert_bytes(bytes):
    """
    Convert bytes into human readable
    """
    bytes = float(bytes)
    if bytes >= 1099511627776:
        terabytes = bytes / 1099511627776
        size = '%.2fT' % terabytes
    elif bytes >= 1073741824:
        gigabytes = bytes / 1073741824
        size = '%.2fG' % gigabytes
    elif bytes >= 1048576:
        megabytes = bytes / 1048576
        size = '%.2fM' % megabytes
    elif bytes >= 1024:
        kilobytes = bytes / 1024
        size = '%.2fK' % kilobytes
    else:
        size = '%.2fb' % bytes
    return size


def list_chunks(l, n):
    """
    Return a list of chunks
    :param l: List
    :param n: int The number of items per chunk
    :return: List
    """
    if n < 1:
        n = 1
    return [l[i:i + n] for i in range(0, len(l), n)]

def any_in_string(l, s):
    """
    Check if any items in a list is in a string
    :params l: dict
    :params s: string
    :return bool:
    """
    return any([i in l for i in l if i in s])

def add_path_to_jinja(flask_app, path):
    """
    To add path to jinja so it can be loaded
    :param flask_app:
    :param path:
    :return:
    """
    template_dir = path
    my_loader = jinja2.ChoiceLoader([
        flask_app.jinja_loader,
        jinja2.FileSystemLoader(template_dir)
    ])
    flask_app.jinja_loader = my_loader

# ------------------------------------------------------------------------------
# Background multi processing and threading
# Use the decorators below for background processing

def bg_process(func):
    """
    A multiprocess decorator
    :param func:
    :return:
    """
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        p = multiprocessing.Process(target=func, args=args, kwargs=kwargs)
        p.start()
    return wrapper

def bg_thread(func):
    """
    A threading decorator
    :param func:
    :return:
    """
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        p = threading.Thread(target=func, args=args, kwargs=kwargs)
        p.start()
    return wrapper


def connect_redis(dsn):
    """
    Return the redis connection
    :param dsn: The dsn url
    :return: Redis
    """
    import redis
    return redis.StrictRedis.from_url(url=dsn)



"""
Maintenance Page
"""

from pylot import Pylot



def maintenance_page(template=None):
    """
    Create the Maintenance view
    Must be instantiated

    import maintenance_view
    MaintenanceView = maintenance_view()

    :param view_template: The directory containing the view pages
    :return:
    """
    if not template:
        template = "Pylot/MaintenancePage/index.html"

    class Maintenance(Pylot):
        @classmethod
        def register(cls, app, **kwargs):
            super(cls, cls).register(app, **kwargs)

            if cls.get_config__("MAINTENANCE_ON"):
                @app.before_request
                def on_maintenance():
                    return cls.render(layout=template), 503
    return Maintenance

MaintenanceV = maintenance_page()



import os
import pkg_resources
import functools
import inspect
from pylot import (Pylot,
                   utils,
                   abort)
from flask_login import current_user
from flask_assets import (Environment, Bundle)

# Views
from contact_page import contact_page
from error_page import error_page
from maintenance_page import maintenance_page
from post_admin import post_admin
from post_reader import post_reader
from user_account import user_account
from user_admin import user_admin


def _register(app):
    def setup_config(app):
        if app.config.get("APP_NAME"):
            Pylot.set_context__(APP_NAME=app.config.get("APP_NAME"))
        if app.config.get("APP_VERSION"):
            Pylot.set_context__(APP_VERSION=app.config.get("APP_VERSION"))

        # OAUTH LOGIN
        if app.config.get("LOGIN_OAUTH_ENABLE"):
            _sl = app.config.get("LOGIN_OAUTH_CREDENTIALS")
            if _sl and isinstance(_sl, dict):
                client_ids = {}
                buttons = []
                for name, prop in _sl.items():
                    if isinstance(prop, dict):
                        if prop["ENABLE"]:
                            _name = name.lower()
                            client_ids[_name] = prop["CLIENT_ID"]
                            buttons.append(_name)

            Pylot.set_context__(LOGIN_OAUTH_ENABLED=True,
                           LOGIN_OAUTH_CLIENT_IDS=client_ids,
                           LOGIN_OAUTH_BUTTONS=buttons)

    def register_templates(app):
        """
        """
        _name = __name__
        _name = ".".join(_name.split(".")[:-1])
        path = pkg_resources.resource_filename(_name, "templates")

        utils.add_path_to_jinja(app, path)

        # Register Assets
        _dir_ = os.path.dirname(__file__)
        _dir_ = os.path.dirname(_dir_)
        env = Pylot.assets
        env.load_path = [
            Pylot._app.static_folder,
            os.path.join(_dir_, 'static'),
        ]

        env.register(
            'pylot_js',
            Bundle(
                "pylot/js/s3upload.js",
                "pylot/js/hello.js",
                "pylot/js/pylot.js",
                output='pylot.js'
            )
        )
        env.register(
            'pylot_css',
            Bundle(
                'pylot/css/pylot.css',
                'pylot/css/bootstrap-social-btns.css',
                output='pylot.css'
            )
        )

    setup_config(app)
    register_templates(app)
Pylot.bind(_register)


def with_user_roles(roles):
    """
    with_user_roles(roles)

    It allows to check if a user has access to a view by adding the decorator
    with_user_roles([])

    Requires flask-login

    In your model, you must have a property 'role', which will be invoked to
    be compared to the roles provided.

    If current_user doesn't have a role, it will throw a 403

    If the current_user is not logged in will throw a 401

    * Require Flask-Login
    ---
    Usage

    @app.route('/user')
    @login_require
    @with_user_roles(['admin', 'user'])
    def user_page(self):
        return "You've got permission to access this page."
    """
    def wrapper(f):
        @functools.wraps(f)
        def wrapped(*args, **kwargs):
            if current_user.is_authenticated():

                if not hasattr(current_user, "role"):
                    raise AttributeError("<'role'> doesn't exist in login 'current_user'")
                if current_user.role not in roles:
                    return abort(403)
            else:
                return abort(401)
            return f(*args, **kwargs)
        return wrapped
    return wrapper


import re
import warnings
import humanize
import jinja2
import mistune
from pylot import (Pylot, utils)
from flask_kvsession import KVSessionExtension
import ses_mailer
import flask_mail
from flask_cloudstorage import Storage
from flask_recaptcha import ReCaptcha
from flask_seasurf import SeaSurf
import flask_cache
from six.moves.urllib.parse import urlparse

class Mailer(object):
    """
    A simple wrapper to switch between SES-Mailer and Flask-Mail based on config
    """
    mail = None
    provider = None
    app = None

    def init_app(self, app):

        self.app = app
        provider = app.config.get("MAILER_PROVIDER", None)

        if provider:
            self.provider = provider.upper()

            if self.provider == "SES":
                class _App(object):
                    config = {
                        "SES_AWS_ACCESS_KEY": app.config.get("MAILER_SES_ACCESS_KEY"),
                        "SES_AWS_SECRET_KEY": app.config.get("MAILER_SES_SECRET_KEY"),
                        "SES_SENDER": app.config.get("MAILER_SENDER"),
                        "SES_REPLY_TO": app.config.get("MAILER_REPLY_TO"),
                        "SES_TEMPLATE": app.config.get("MAILER_TEMPLATE"),
                        "SES_TEMPLATE_CONTEXT": app.config.get("MAILER_TEMPLATE_CONTEXT")
                    }
                _app = _App()
                self.mail = ses_mailer.Mail(app=_app)

            elif self.provider == "SMTP":
                uri = app.config.get("MAILER_SMTP_URI", None)
                if uri is None:
                    raise ValueError("<Pylot:Component:Mailer: MAILER_SMTP_URI is empty'")

                parse_uri = urlparse(uri)
                if "smtp" not in parse_uri.scheme:
                    raise ValueError("<Pylot:Component:Mailer: MAILER_SMTP_URI must start with 'smtp://'")

                class _App(object):
                    config = {
                        "MAIL_SERVER": parse_uri.hostname,
                        "MAIL_USERNAME": parse_uri.username,
                        "MAIL_PASSWORD": parse_uri.password,
                        "MAIL_PORT": parse_uri.port,
                        "MAIL_USE_TLS": True if "tls" in parse_uri.scheme else False,
                        "MAIL_USE_SSL": True if "ssl" in parse_uri.scheme else False,
                        "MAIL_DEFAULT_SENDER": app.config.get("MAILER_SENDER"),
                        "TESTING": app.config.get("TESTING"),
                        "DEBUG": app.config.get("DEBUG")
                    }
                    debug = app.config.get("DEBUG")
                    testing = app.config.get("TESTING")

                _app = _App()
                self.mail = flask_mail.Mail(app=_app)

            else:
                raise warnings.warn("<Pylot:Component:Mailer invalid provider '%s'>" % provider)

    def send(self, to, subject, body, reply_to=None, **kwargs):
        """
        Send simple message
        """
        if self.provider == "SES":
            self.mail.send(to=to,
                           subject=subject,
                           body=body,
                           reply_to=reply_to,
                           **kwargs)

        elif self.provider == "SMTP":
            print body
            msg = flask_mail.Message(recipients=[to] if not isinstance(to, list) else to,
                                     subject=subject,
                                     body=body,
                                     reply_to=reply_to,
                                     sender=self.app.config.get("MAILER_SENDER"))
            self.mail.send(msg)

    def send_template(self, template, to, reply_to=None, **context):
        """
        Send Template message
        """
        if self.provider == "SES":
            self.mail.send_template(template=template,
                                    to=to,
                                    reply_to=reply_to,
                                    **context)

        elif self.provider == "SMTP":
            _template = self.app.config.get("MAILER_TEMPLATE", None)
            template_context = self.app.config.get("MAILER_TEMPLATER_CONTEXT")

            ses_mail = ses_mailer.Mail(template=_template,
                                       template_context=template_context)
            data = ses_mail.parse_template(template=template, **context)

            msg = flask_mail.Message(recipients=[to] if not isinstance(to, list) else to,
                                     subject=data["subject"],
                                     body=data["body"],
                                     reply_to=reply_to,
                                     sender=self.app.config.get("MAILER_SENDER")
                                     )
            self.mail.send(msg)

class Session(object):
    """
    Use KVSession
    """
    def __init__(self, app):
        store = None
        uri = app.config.get("SESSION_URI")
        if uri:
            parse_uri = urlparse(uri)
            scheme = parse_uri.scheme
            username = parse_uri.username
            password = parse_uri.password
            hostname = parse_uri.hostname
            port = parse_uri.port
            bucket = parse_uri.path.strip("/")

            if "redis" in scheme:
                import redis
                from simplekv.memory.redisstore import RedisStore
                conn = redis.StrictRedis.from_url(url=uri)
                store = RedisStore(conn)
            elif "s3" in scheme or "google_storage" in scheme:
                from simplekv.net.botostore import BotoStore
                import boto
                if "s3" in scheme:
                    _con_fn = boto.connect_s3
                else:
                    _con_fn = boto.connect_gs
                conn = _con_fn(username, password)
                _bucket = conn.create_bucket(bucket)
                store = BotoStore(_bucket)
            elif "memcache" in scheme:
                import memcache
                from simplekv.memory.memcachestore import MemcacheStore
                host_port = "%s:%s" % (hostname, port)
                conn = memcache.Client(servers=[host_port])
                store = MemcacheStore(conn)
            elif "sql" in scheme:
                from simplekv.db.sql import SQLAlchemyStore
                from sqlalchemy import create_engine, MetaData
                engine = create_engine(uri)
                metadata = MetaData(bind=engine)
                store = SQLAlchemyStore(engine, metadata, 'kvstore')
                metadata.create_all()
            else:
                raise ValueError("Invalid Session Store")
        if store:
            KVSessionExtension(store, app)

mailer = Mailer()
cache = flask_cache.Cache()
storage = Storage()
recaptcha = ReCaptcha()
csrf = SeaSurf()

Pylot.bind(Session)
Pylot.bind(mailer.init_app)
Pylot.bind(storage.init_app)
Pylot.bind(cache.init_app)
Pylot.bind(recaptcha.init_app)
Pylot.bind(csrf.init_app)

# ------------------------------------------------------------------------------

def to_date(dt, format="%m/%d/%Y"):
    return "" if not dt else dt.strftime(format)

def strip_decimal(amount):
    return amount.split(".")[0]

def bool_to_yes(b):
    return "Yes" if b is True else "No"

def bool_to_int(b):
    return 1 if b is True else 0

def nl2br(s):
    """
    {{ s|nl2br }}

    Convert newlines into <p> and <br />s.
    """
    if not isinstance(s, basestring):
        s = str(s)
    s = re.sub(r'\r\n|\r|\n', '\n', s)
    paragraphs = re.split('\n{2,}', s)
    paragraphs = ['<p>%s</p>' % p.strip().replace('\n', '<br />') for p in paragraphs]
    return '\n\n'.join(paragraphs)


jinja2.filters.FILTERS.update({
    "currency": utils.to_currency,
    "strip_decimal": strip_decimal,
    "date": to_date,
    "int": int,
    "slug": utils.slug,
    "intcomma": humanize.intcomma,
    "intword": humanize.intword,
    "naturalday": humanize.naturalday,
    "naturaldate": humanize.naturaldate,
    "naturaltime": humanize.naturaltime,
    "naturalsize": humanize.naturalsize,
    "bool_to_yes": bool_to_yes,
    "bool_to_int": bool_to_int,
    "nl2br": nl2br,
    "markdown": mistune.markdown
})






"""
Pylot

"""

import os
import datetime
import inspect
from werkzeug.contrib.fixers import ProxyFix
from flask_classy import (FlaskView,
                          route)
from flask import (Flask,
                   abort,
                   redirect,
                   request,
                   render_template,
                   flash,
                   url_for,
                   jsonify,
                   session)

from flask_assets import Environment
import utils

# ------------------------------------------------------------------------------
import pkginfo
NAME = pkginfo.NAME
__version__ = pkginfo.VERSION
__author__ = pkginfo.AUTHOR
__license__ = pkginfo.LICENSE
__copyright__ = pkginfo.COPYRIGHT

# ------------------------------------------------------------------------------

class Pylot(FlaskView):
    """ Pylot """

    LAYOUT = "layout.html"  # The default layout
    assets = None
    _app = None
    _bind = set()
    _context = dict(
        APP_NAME="",
        APP_VERSION="",
        YEAR=datetime.datetime.now().year,
        GOOGLE_ANALYTICS_ID=None,
        LOGIN_ENABLED=False,
        LOGIN_OAUTH_ENABLED=False,
        LOGIN_OAUTH_CLIENT_IDS=[],
        LOGIN_OAUTH_BUTTONS=[],
        META=dict(
            title="",
            description="",
            url="",
            image="",
            site_name="",
            object_type="",
            locale="",
            keywords=[],
            use_opengraph=True,
            use_googleplus=True,
            use_twitter=True
        )
    )

    @classmethod
    def init(cls, flask_or_import_name, directory=None, config=None):
        """
        Allow to register all subclasses of Pylot
        So we call it once initiating
        :param flask_or_import_name: Flask instance or import name -> __name__
        :param directory: The directory containing your project's Views, Templates and Static
        :param config: string of config object. ie: "app.config.Dev"
        """
        if isinstance(flask_or_import_name, Flask):
            app = flask_or_import_name
        else:
            app = Flask(flask_or_import_name)

        app.wsgi_app = ProxyFix(app.wsgi_app)

        if config:
            app.config.from_object(config)

        if directory:
            app.template_folder = directory + "/templates"
            app.static_folder = directory + "/static"

        cls._app = app
        cls.assets = Environment(cls._app)

        for _app in cls._bind:
            _app(cls._app)

        for subcls in cls.__subclasses__():
            subcls.register(cls._app)

        return cls._app

    @classmethod
    def bind(cls, kls):
        """
        To bind middlewares that needs the 'app' object to init
        Bound middlewares will be assigned on cls.init()
        """
        if not hasattr(kls, "__call__"):
            raise TypeError("From Pylot.bind: '%s' is not callable" % kls)
        cls._bind.add(kls)
        return kls

    @classmethod
    def render(cls, data={}, view_template=None, layout=None, **kwargs):
        """
        To render data to the associate template file of the action view
        :param data: The context data to pass to the template
        :param view_template: The file template to use. By default it will map the classname/action.html
        :param layout: The body layout, must contain {% include __view_template__ %}
        """
        if not view_template:
            stack = inspect.stack()[1]
            module = inspect.getmodule(cls).__name__
            module_name = module.split(".")[-1]
            action_name = stack[3]      # The method being called in the class
            view_name = cls.__name__    # The name of the class without View

            if view_name.endswith("View"):
                view_name = view_name[:-4]
            view_template = "%s/%s.html" % (view_name, action_name)

        data = data if data else dict()
        data["__"] = cls._context if cls._context else {}
        if kwargs:
            data.update(kwargs)

        data["__view_template__"] = view_template

        return render_template(layout or cls.LAYOUT, **data)

    @classmethod
    def extends__(cls, kls):
        """
        A view decorator to extend another view class or function to itself
        It will inherit all its methods and propeties and use them on itself

        -- EXAMPLES --

        class Index(Pylot):
            pass

        index = Index()

        ::-> As decorator on classes ::
        @index.extends__
        class A(object):
            def hello(self):
                pass

        @index.extends__
        class C()
            def world(self):
                pass

        ::-> Decorator With function call ::
        @index.extends__
        def hello(self):
            pass

        """
        if inspect.isclass(kls):
            for _name, _val in kls.__dict__.items():
                if not _name.startswith("__"):
                    setattr(cls, _name, _val)
        elif inspect.isfunction(kls):
            setattr(cls, kls.__name__, kls)
        return cls

    @classmethod
    def set_context__(cls, **kwargs):
        """
        Assign a global view context to be used in the template
        :params **kwargs:
        """
        cls._context.update(kwargs)

    @classmethod
    def get_config__(cls, key, default=None):
        """
        Shortcut to access the config in your class
        :param key: The key to access
        :param default: The default value when None
        :returns mixed:
        """
        return cls._app.config.get(key, default)

    @classmethod
    def set_meta__(cls, **kwargs):
        """
        Meta allows you to add meta data to site
        :params **kwargs:

        meta keys we're expecting:
            title (str)
            description (str)
            url (str) (Will pick it up by itself if not set)
            image (str)
            site_name (str) (but can pick it up from config file)
            object_type (str)
            keywords (list)
            locale (str)

            **Boolean By default these keys are True
            use_opengraph
            use_twitter
            use_googleplus

        """
        _name_ = "META"
        meta_data = cls._context.get(_name_, {})
        for k, v in kwargs.items():
            # Prepend/Append string
            if (k.endswith("__prepend") or k.endswith("__append")) \
                    and isinstance(v, str):
                k, position = k.split("__", 2)
                _v = meta_data.get(k, "")
                if position == "prepend":
                    v += _v
                elif position == "append":
                    v = _v + v
            if k == "keywords" and not isinstance(k, list):
                raise ValueError("Meta keyword must be a list")
            meta_data[k] = v
        cls.set_context__(_name_=meta_data)

    @classmethod
    def flash_success__(cls, message):
        """ Set a flash success message """
        flash(message, "success")

    @classmethod
    def flash_error__(cls, message):
        """ Set a flash error message """
        flash(message, "error")


class AppError(Exception):
    """ For Application based error """
    pass



import os

from pylot import (Pylot,
                   utils,
                   route,
                   abort,
                   redirect,
                   request,
                   url_for,
                   jsonify,
                   session,
                   AppError)

from pylot.component import (mailer,
                             storage,
                             recaptcha)
from flask_login import (LoginManager,
                         login_required,
                         login_user,
                         logout_user,
                         current_user)



def user_admin(model, **kwargs):

    def wrapper(view):

        Pylot.set_context__(COMPONENT_USER_ADMIN=True)
        User = model.UserStruct.User
        Role = model.UserStruct.Role
        RoleRole = model.UserStruct.RoleRole

        template_dir = kwargs["template_dir"] if "template_dir" \
                                                 in kwargs else "Pylot/UserAdmin"
        template_page = template_dir + "/%s.html"

        @view.extends__
        class UserAdmin(object):
            route_base = "user-admin"
            per_page = 25

            @classmethod
            def _validate_admin_roles(cls, user):
                admin = current_user

            @classmethod
            def _user_roles_options(cls):
                return [(r.id, r.name) for r in Role.all().order_by(Role.level.desc())]

            @route("/", endpoint="UserAdmin:index")
            def index(self):
                self.set_meta__(title="Users - User Admin")
                per_page = self.get_config__("PAGINATION_PER_PAGE", 25)

                page = request.args.get("page", 1)
                include_deleted = True if request.args.get("include-deleted") == "y" else False
                name = request.args.get("name")
                email = request.args.get("email")
                role = request.args.get("role")
                sorting = request.args.get("sorting", "name__asc")

                users = User.all(include_deleted=include_deleted)

                if name:
                    users = users.filter(User.name.contains(name))
                if email:
                    users = users.filter(User.email.contains(email))
                if role:
                    users = users.join(RoleRole)\
                        .join(Role)\
                        .filter(Role.id == role)
                if sorting and "__" in sorting:
                    col, dir = sorting.split("__", 2)
                    users = users.order_by(getattr(User, col) + " %s" % dir)

                users = users.paginate(page=page, per_page=per_page)

                sorting = [("name__asc", "Name ASC"),
                           ("name__desc", "Name DESC"),
                           ("email__asc", "Email ASC"),
                           ("email__DESC", "Email DESC"),
                           ("created_at__asc", "Signup ASC"),
                           ("created_at__desc", "Signup Desc"),
                           ("last_login__asc", "Login ASC"),
                           ("last_login__desc", "Login Desc"),
                           ]
                return self.render(user_roles_options=self._user_roles_options(),
                                   sorting_options=sorting,
                                   users=users,
                                   search_query={
                                       "include-deleted": request.args.get("include-deleted", "n"),
                                       "role": int(request.args.get("role")) if request.args.get("role") else "",
                                       "status": request.args.get("status"),
                                       "name": request.args.get("name", ""),
                                       "email": request.args.get("email", ""),
                                       "sorting": request.args.get("sorting")},
                                   view_template=template_page % "index")

            @route("/<id>", endpoint="UserAdmin:get")
            def get(self, id):
                self.set_meta__(title="User Info - Users Admin")
                user = User.get(id, include_deleted=True)
                if not user:
                    abort(404, "User doesn't exist")

                user_roles = [r.id for r in user.roles]
                user_roles_name = [r.name for r in user.roles]

                return self.render(user=user,
                                   user_roles_name=user_roles_name,
                                   user_roles=user_roles,
                                   user_roles_options=self._user_roles_options(),
                                   view_template=template_page % "get")

            @route("/post", methods=["POST"], endpoint="UserAdmin:post")
            def post(self):
                try:
                    id = request.form.get("id")
                    user = User.get(id, include_deleted=True)
                    if not user:
                        self.flash_error__("Can't change user info. Invalid user")
                        return redirect(url_for("UserAdmin:index"))

                    email = request.form.get("email", "").strip()
                    name = request.form.get("name")
                    user_roles = request.form.getlist("user_roles")
                    action = request.form.get("action")

                    if action == "activate":
                        user.update(active=True)
                        self.flash_success__("User has been ACTIVATED")
                    elif action == "deactivate":
                        user.update(active=False)
                        self.flash_success__("User is now DEACTIVATED")
                    elif action == "delete":
                        user.delete()
                        self.flash_success__("User has been deleted")
                    elif action == "undelete":
                        user.delete(False)
                        self.flash_success__("User is now active")
                    else:
                        if email and email != user.email:
                            if not utils.is_valid_email(email):
                                raise UserWarning("Invalid email address '%s'" % email)
                            else:
                                if User.get_by_email(email):
                                    raise UserWarning("Email exists already '%s'" % email)
                                user.update(email=email)
                        if name != user.name:
                            user.update(name=name)

                        if user_roles:
                            user.update_roles(map(int, user_roles))

                        self.flash_success__("User's Info updated successfully!")
                except Exception as ex:
                    self.flash_error__("Error: %s " % ex.message)
                return redirect(url_for("UserAdmin:get", id=id))

            @route("reset-password", methods=["POST"], endpoint="UserAdmin:reset_password")
            def reset_password(self):
                """
                Reset the password
                :returns string: The new password string
                """
                try:
                    id = request.form.get("id")
                    user = User.get(id)
                    if not user:
                        raise AppError("Invalid User")

                    method_ = self.get_config__("LOGIN_RESET_PASSWORD_METHOD", "").upper()
                    new_password = None
                    if method_ == "TOKEN":
                        token = user.set_reset_password_token()
                        url = url_for("UserAccount:reset_password_token",
                                      token=token,
                                      _external=True)
                    else:
                        new_password = user.set_random_password()
                        url = url_for("UserAccount:login", _external=True)

                    mailer.send_template("reset-password.txt",
                                         method_=method_,
                                         to=user.email,
                                         name=user.email,
                                         url=url,
                                         new_password=new_password)

                    self.flash_success__("Password Reset instruction is sent to email")
                except Exception as ex:
                    self.flash_error__("Error: %s " % ex.message)
                return redirect(url_for("UserAdmin:get", id=id))

            @route("create", methods=["POST"], endpoint="UserAdmin:create")
            def create(self):
                try:
                    email = request.form.get("email")
                    name = request.form.get("name")
                    user_roles = request.form.getlist("user_roles")
                    if not name:
                        raise AppError("Name is required")
                    elif not email:
                        raise AppError("Email is required")
                    elif not utils.is_valid_email(email):
                        raise AppError("Invalid email address")
                    if User.get_by_email(email):
                        raise AppError("Email '%s' exists already" % email)
                    else:
                        user = User.new(email=email, name=name, signup_method="EMAIL - FROM ADMIN")
                        if user:
                            self.flash_success__("User created successfully!")

                            if user_roles:
                                user.update_roles(map(int, user_roles))

                            return redirect(url_for("UserAdmin:get", id=user.id))
                        else:
                            raise AppError("Couldn't create new user")
                except Exception as ex:
                    self.flash_error__("Error: %s" % ex.message)
                return redirect(url_for("UserAdmin:index"))

            @route("roles", methods=["GET", "POST"], endpoint="UserAdmin:roles")
            def roles(self):
                """
                Only admin and super admin can add/remove roles
                RESTRICTED ROLES CAN'T BE CHANGED
                """
                roles_rage_max = 11
                if request.method == "POST":
                    try:
                        id = request.form.get("id")
                        name = request.form.get("name")
                        level = request.form.get("level")
                        action = request.form.get("action")

                        if name and level:
                            level = int(level)
                            name = name.upper()
                            _levels = [r[0] for r in Role.PRIMARY]
                            _names = [r[1] for r in Role.PRIMARY]
                            if level in _levels or name in _names:
                                raise AppError("Can't modify PRIMARY Roles - name: %s, level: %s " % (name, level))
                            else:
                                if id:
                                    role = Role.get(id)
                                    if role:
                                        if action == "delete":
                                            role.delete()
                                            self.flash_success__("Role '%s' deleted successfully!" % role.name)
                                        elif action == "update":
                                            if role.level != level and Role.get_by_level(level):
                                                raise AppError("Role Level '%s' exists already" % level)
                                            elif role.name != name and Role.get_by_name(name):
                                                raise AppError("Role Name '%s'  exists already" % name)
                                            else:
                                                role.update(name=name, level=level)
                                                self.flash_success__("Role '%s (%s)' updated successfully" % (name, level))
                                    else:
                                        raise AppError("Role doesn't exist")
                                else:
                                    if Role.get_by_level(level):
                                        raise AppError("Role Level '%s' exists already" % level)
                                    elif Role.get_by_name(name):
                                        raise AppError("Role Name '%s'  exists already" % name)
                                    else:
                                        Role.new(name=name, level=level)
                                        self.flash_success__("New Role '%s (%s)' addedd successfully" % (name, level))
                    except Exception as ex:
                        self.flash_error__("Error: %s" % ex.message)
                    return redirect(url_for("UserAdmin:roles"))
                else:
                    self.set_meta__(title="User Roles - Users Admin")
                    roles = Role.all().order_by(Role.level.desc())

                    allocated_levels = [r.level for r in roles]
                    levels_options = [(l, l) for l in range(1, roles_rage_max) if l not in allocated_levels]

                    return self.render(roles=roles,
                                       levels_options=levels_options,
                                       view_template=template_page % "roles")
        return view
    return wrapper



import datetime
from pylot import utils
from active_sqlalchemy import SQLAlchemy


class ModelError(Exception):
    pass


# The user_model create a fully built model with social signin
def user_struct(db):

    class UserRole(db.Model):
        name = db.Column(db.String(75), index=True)
        level = db.Column(db.Integer, index=True)

        # Primary Roles
        PRIMARY = [(99, "SUPERADMIN"),
                   (98, "ADMIN"),
                   (1, "USER")]

        @classmethod
        def new(cls, name, level):
            name = name.upper()
            role = cls.get_by_name(name)
            if not role:
                role = cls.create(name=name, level=level)
            return role

        @classmethod
        def get_by_name(cls, name):
            name = name.upper()
            return cls.all().filter(cls.name == name).first()

        @classmethod
        def get_by_level(cls, level):
            return cls.all().filter(cls.level == level).first()

    class User(db.Model):

        email = db.Column(db.String(75), index=True, unique=True)
        email_confirmed = db.Column(db.Boolean, default=False)
        password_hash = db.Column(db.String(250))
        require_password_change = db.Column(db.Boolean, default=False)
        reset_password_token = db.Column(db.String(100), index=True)
        reset_password_token_expiration = db.Column(db.DateTime)
        name = db.Column(db.String(250))
        profile_pic_url = db.Column(db.String(250))
        signup_method = db.Column(db.String(250))
        active = db.Column(db.Boolean, default=True, index=True)
        last_login = db.Column(db.DateTime)
        last_visited = db.Column(db.DateTime)
        roles = db.relationship(UserRole, secondary="user_role_role")

        # ------ FLASK-LOGIN REQUIRED METHODS ----------------------------------

        def is_active(self):
            return self.active

        def get_id(self):
            return self.id

        def is_authenticated(self):
            return True

        def is_anonymous(self):
            return False

        # ---------- END FLASK-LOGIN REQUIREMENTS ------------------------------

        @classmethod
        def get_by_email(cls, email):
            """
            Find by email. Useful for logging in users
            """
            return cls.all().filter(cls.email == email).first()

        @classmethod
        def get_by_token(cls, token):
            """
            Find by email. Useful for logging in users
            """
            user = cls.all().filter(cls.reset_password_token == token).first()
            if user:
                now = datetime.datetime.now()
                if user.require_password_change is True \
                        and user.reset_password_token_expiration > now:
                    return user
                else:
                    user.clear_reset_password_token()
            else:
                return None

        @classmethod
        def new(cls, email, password=None, role="USER", **kwargs):
            """
            Register a new user
            """
            user = cls.get_by_email(email)
            if user:
                raise ModelError("User exists already")
            user = cls.create(email=email)
            if password:
                user.set_password(password)
            if kwargs:
                user.update(**kwargs)
            if role:
                role_ = UserRole.get_by_name(role.upper())
                if role_:
                    user.update_roles([role_.id])

            return user

        def password_matched(self, password):
            """
            Check if the password matched the hash
            :returns bool:
            """
            return utils.verify_encrypted_string(password, self.password_hash)

        def set_password(self, password):
            """
            Encrypt the password and save it in the DB
            """
            self.update(password_hash=utils.encrypt_string(password))

        def set_random_password(self):
            """
            Set a random password, saves it and return the readable string
            :returns string:
            """
            password = utils.generate_random_string()
            self.set_password(password)
            return password

        def set_reset_password_token(self, expiration=24):
            """
            Generate password reset token
            It returns the token generated
            """
            expiration = datetime.datetime.now() + datetime.timedelta(hours=expiration)
            while True:
                token = utils.generate_random_string(32).lower()
                if not User.all().filter(User.reset_password_token == token).first():
                    break
            self.update(require_password_change=True,
                        reset_password_token=token,
                        reset_password_token_expiration=expiration)
            return token

        def clear_reset_password_token(self):
            self.update(require_password_change=False,
                        reset_password_token=None,
                        reset_password_token_expiration=None)

        def set_require_password_change(self, req=True):
            self.update(require_password_change=req)

        def update_last_login(self):
            self.update(last_login=datetime.datetime.now())

        def update_last_visited(self):
            self.update(last_visited=datetime.datetime.now())

        @classmethod
        def oauth_register(cls, provider, provider_user_id=None,
                          email=None, name=None, image_url=None,
                          **kwargs):
            """
            Register
            :param provider:
            :param provider_user_id:
            :param email:
            :param name:
            :param image_url:
            :param kwargs:
            :return:
            """
            oal = UserOauthLogin
            oauthuser = oal.all()\
                .filter(oal.provider == provider)\
                .filter(oal.provider_user_id == provider_user_id)\
                .first()
            if oauthuser:
                return oauthuser.user
            else:
                if not email:
                    raise ModelError("Email is missing")

                data = {
                    "provider": provider,
                    "provider_user_id": provider_user_id,
                    "email": email,
                    "name": name,
                    "image_url": image_url
                }

                user = cls.get_by_email(email)
                if user:
                    data.update({"user_id": user.id})
                    oal.create(**data)
                    return user
                else:
                    user = cls.new(email=email,
                                    name=name,
                                    profile_pic_url=image_url,
                                    signin_method=provider)
                    data.update({"user_id": user.id})
                    oal.create(**data)
                    return user

        def oauth_connect(self, provider, provider_user_id=None,
                          email=None, name=None, image_url=None,
                          **kwargs):
            """
            Connect an account an OAUTH
            :param provider:
            :param provider_user_id:
            :param email:
            :param name:
            :param image_url:
            :param kwargs:
            :return:
            """
            oal = UserOauthLogin
            oauthuser = oal.all()\
                .filter(oal.provider == provider)\
                .filter(oal.provider_user_id == provider_user_id)\
                .first()
            if oauthuser:
                if oauthuser.user_id == self.id:
                    return self
                else:
                    raise ModelError("Account is already linked to another user")
            else:
                data = {
                    "provider": provider,
                    "provider_user_id": provider_user_id,
                    "email": email,
                    "name": name,
                    "image_url": image_url,
                    "user_id": self.id
                }
                oal.create(**data)
                return self

        def update_roles(self, roles_list):
            r_list = [r.id for r in self.roles]
            del_roles = list(set(r_list) - set(roles_list))
            new_roles = list(set(roles_list) - set(r_list))
            for dc in del_roles:
                UserRoleRole.remove(user_id=self.id, role_id=dc)
            for nc in new_roles:
                UserRoleRole.add(user_id=self.id, role_id=nc)

    class UserRoleRole(db.Model):
        user_id = db.Column(db.Integer, db.ForeignKey(User.id))
        role_id = db.Column(db.Integer, db.ForeignKey(UserRole.id))

        @classmethod
        def add(cls, user_id, role_id):
            r = cls.all().filter(cls.user_id == user_id)\
                .filter(cls.role_id == role_id)\
                .first()
            if not r:
                cls.create(user_id=user_id, role_id=role_id)

        @classmethod
        def remove(cls, user_id, role_id):
            r = cls.all().filter(cls.user_id == user_id)\
                .filter(cls.role_id == role_id)\
                .first()
            if r:
                r.delete(hard_delete=True)

    class UserOauthLogin(db.Model):
        user_id = db.Column(db.Integer, db.ForeignKey(User.id))
        provider = db.Column(db.String(50), index=True)
        provider_user_id = db.Column(db.String(250))
        name = db.Column(db.String(250))
        email = db.Column(db.String(250))
        image_url = db.Column(db.String(250))
        access_token = db.Column(db.String(250))
        secret = db.Column(db.String(250))
        profile_url = db.Column(db.String(250))
        user = db.relationship(User, backref="oauth_logins")

    return utils.to_struct(User=User,
                           Role=UserRole,
                           RoleRole=UserRoleRole,
                           OauthLogin=UserOauthLogin)

# File assets manager
def cloud_storage_struct(db):
    class CloudStorageObject(db.Model):
        parent_id = db.Column(db.Integer, index=True)
        user_id = db.Column(db.Integer, index=True)
        provider = db.Column(db.String(250))
        container = db.Column(db.String(250))
        local_path = db.Column(db.String(250))
        name = db.Column(db.String(250))
        description = db.Column(db.String(250))
        extension = db.Column(db.String(10), index=True)
        type = db.Column(db.String(250), index=True)
        size = db.Column(db.Integer, default=None)
        width = db.Column(db.Integer, default=None)
        object_hash = db.Column(db.String(250))
        object_url = db.Column(db.String(250))
        is_private = db.Column(db.Boolean, index=True, default=False)

    return utils.to_struct(StorageObject=CloudStorageObject)


def post_struct(UserStruct):
    """
    Post Model
    :param UserStruct:
    """

    db = UserStruct.User.db

    class SlugNameMixin(object):
        name = db.Column(db.String(250))
        slug = db.Column(db.String(250), index=True, unique=True)

        @classmethod
        def get_by_slug(cls, slug):
            """
            Return a post by slug
            """
            return cls.all().filter(cls.slug == slug).first()

        @classmethod
        def new(cls, name, slug=None):
            slug = utils.slug(name if not slug else slug)
            return cls.create(name=name, slug=slug)

        def rename(self, name, slug=None):
            slug = utils.slug(name if not slug else slug)
            return self.update(name=name, slug=slug)

    class PostType(SlugNameMixin, db.Model):
        """
        Types
        """
        @property
        def total_posts(self):
            return Post.all().filter(Post.type_id == self.id).count()

    class PostCategory(SlugNameMixin, db.Model):
        """
        Category
        """
        @property
        def total_posts(self):
            return PostPostCategory.all()\
                .filter(PostPostCategory.category_id == self.id)\
                .count()

    class PostTag(db.Model):
        """
        Tag
        """
        name = db.Column(db.String(250), index=True)

    class PostPostTag(db.Model):
        """
        PostPostTag
        """
        post_id = db.Column(db.Integer, db.ForeignKey("post.id"))
        tag_id = db.Column(db.Integer, db.ForeignKey(PostTag.id))

        @classmethod
        def add(cls, post_id, tag_id):
            c = cls.all().filter(cls.post_id == post_id)\
                .filter(cls.tag_id == tag_id)\
                .first()
            if not c:
                cls.create(post_id=post_id, tag_id=tag_id)

        @classmethod
        def remove(cls, post_id, tag_id):
            c = cls.all().filter(cls.post_id == post_id)\
                .filter(cls.tag_id == tag_id)\
                .first()
            if c:
                c.delete(hard_delete=True)

    class PostPostCategory(db.Model):
        post_id = db.Column(db.Integer, db.ForeignKey("post.id"))
        category_id = db.Column(db.Integer, db.ForeignKey(PostCategory.id))

        @classmethod
        def add(cls, post_id, category_id):
            c = cls.all().filter(cls.post_id == post_id)\
                .filter(cls.category_id == category_id)\
                .first()
            if not c:
                cls.create(post_id=post_id, category_id=category_id)

        @classmethod
        def remove(cls, post_id, category_id):
            c = cls.all().filter(cls.post_id == post_id)\
                .filter(cls.category_id == category_id)\
                .first()
            if c:
                c.delete(hard_delete=True)

    class Post(db.Model):

        user_id = db.Column(db.Integer, db.ForeignKey(UserStruct.User.id))
        type_id = db.Column(db.Integer, db.ForeignKey(PostType.id))
        parent_id = db.Column(db.Integer)
        revision_id = db.Column(db.Integer)
        title = db.Column(db.String(250))
        slug = db.Column(db.String(250), index=True)
        content = db.Column(db.Text)
        excerpt = db.Column(db.Text)
        is_public = db.Column(db.Boolean, index=True, default=False)
        is_sticky = db.Column(db.Boolean, index=True, default=False)
        is_published = db.Column(db.Boolean, index=True, default=True)
        is_draft = db.Column(db.Boolean, index=True, default=False)
        is_revision = db.Column(db.Boolean, default=False)
        published_date = db.Column(db.DateTime)
        author = db.relationship(UserStruct.User, backref="posts")
        type = db.relationship(PostType, backref="posts")
        categories = db.relationship(PostCategory,
                                     secondary=PostPostCategory.__table__.name)
        tags = db.relationship(PostTag,
                                     secondary=PostPostTag.__table__.name)

        @classmethod
        def new(cls, title, **kwargs):
            """
            Insert a new post
            """
            published_date = None
            is_revision = False
            is_published = False
            is_draft = False
            is_public = kwargs["is_public"] if "is_public" in kwargs else True
            parent_id = int(kwargs["parent_id"]) if "parent_id" in kwargs else None
            if "is_revision" in kwargs and kwargs["is_revision"] is True:
                if not parent_id:
                    raise ModelError("'parent_id' is missing for revision")
                is_revision =True
                is_public = False
            elif "is_draft" in kwargs and kwargs["is_draft"] is True:
                is_draft = True
                is_public = False
            elif "is_published" in kwargs and kwargs["is_published"] is True:
                is_published = True
                published_date = datetime.datetime.now()

            slug = ""
            if is_published or is_draft:
                slug = cls.create_slug(title if "slug" not in kwargs else kwargs["slug"])

            data = {
                "title": title,
                "slug": slug,
                "content": kwargs["content"] if "content" in kwargs else "",
                "excerpt": kwargs["excerpt"] if "excerpt" in kwargs else "",
                "is_published": is_published,
                "published_date": published_date,
                "is_draft": is_draft,
                "is_revision": is_revision,
                "is_public": is_public,
                "parent_id": parent_id,
                "type_id": kwargs["type_id"] if "type_id" in kwargs else None
            }
            return cls.create(**data)

        @classmethod
        def get_published(cls, id=None, slug=None):
            """
            Get a published post by id or slug
            :param id: The id of the post
            :param slug: str - The slug to look for
            """
            post = None
            if id:
                post = cls.get(id)
            elif slug:
                post = cls.get_by_slug(slug)
            return post if post and post.is_published else None

        @classmethod
        def get_published_by_category_slug(cls, slug):
            """
            Query by category slug
            :return SQLA :
            """
            return cls.all()\
                .join(PostPostCategory)\
                .join(PostCategory)\
                .filter(PostCategory.slug.in_(slug))\
                .filter(cls.is_published == True)

        @classmethod
        def get_published_by_type_slug(cls, slug):
            """
            Query by type slug
            :return SQLA :
            """
            return cls.all()\
                .join(PostType)\
                .filter(PostType.slug.in_(slug))\
                .filter(cls.is_published == True)

        @classmethod
        def create_slug(cls, title):
            slug_counter = 0
            _slug = utils.slug(title).lower()
            while True:
                slug = _slug
                if slug_counter > 0:
                    slug += str(slug_counter)
                    slug_counter += 1
                if not cls.get_by_slug(slug):
                    break
            return slug

        @classmethod
        def get_by_slug(cls, slug):
            """
            Return a post by slug
            """
            return cls.all().filter(cls.slug == slug).first()

        def publish(self, published_date=None):
            if self.is_draft:
                data = {
                    "is_draft": False,
                    "is_published": True,
                    "published_date": published_date or datetime.datetime.now()
                }
                self.update(**data)

        def set_type(self, type_id):
            self.update(type_id=type_id)

        def set_slug(self, title):
            slug = utils.slug(title)
            if title and slug != self.slug:
                slug = self.create_slug(slug)
                self.update(slug=slug)

        def update_categories(self, categories_list):
            cats = PostPostCategory.all()\
                    .filter(PostPostCategory.post_id == self.id)
            cats_list = [c.category_id for c in cats]

            del_cats = list(set(cats_list) - set(categories_list))
            new_cats = list(set(categories_list) - set(cats_list))

            for dc in del_cats:
                PostPostCategory.remove(post_id=self.id, category_id=dc)

            for nc in new_cats:
                PostPostCategory.add(post_id=self.id, category_id=nc)

        @property
        def status(self):
            if self.is_published:
                return "Published"
            elif self.is_draft:
                return "Draft"
            elif self.is_revision:
                return "Revision"

        def delete_revisions(self):
            """
            Delete all revisions
            """
            try:
                Post.all()\
                    .filter(Post.post_id == self.id)\
                    .filter(Post.is_revision == True)\
                    .delete()
                Post.db.commit()
            except Exception as ex:
                Post.db.rollback()

        @property
        def total_revisions(self):
            return Post.all()\
                .filter(Post.post_id == self.id)\
                .filter(Post.is_revision == True)\
                .count()

    class PostFile(db.Model):
        parent_id = db.Column(db.Integer, index=True)
        user_id = db.Column(db.Integer, index=True)
        provider = db.Column(db.String(250))
        container = db.Column(db.String(250))
        name = db.Column(db.String(250))
        description = db.Column(db.String(250))
        extension = db.Column(db.String(10), index=True)
        object_hash = db.Column(db.String(250))
        object_url = db.Column(db.String(250))
        is_private = db.Column(db.Boolean, index=True, default=False)

    return utils.to_struct(Post=Post,
                           Category=PostCategory,
                           Type=PostType,
                           PostCategory=PostPostCategory,
                           Tag=PostTag,
                           PostTag=PostPostTag,
                           File=PostFile
                           )




"""
Your views
"""

from pylot import (Pylot,
                   mailer,
                   storage,
                   route,
                   abort,
                   request,
                   redirect,
                   url_for,
                   jsonify,
                   session,
                   AppError)
from pylot.views import (login_view,
                         contact_view)

from application import model

LoginView = login_view(model.User)
ContactView = contact_view()

# Login
class Login(LoginView, Pylot):
    pass

# Index
class Index(ContactView, Pylot):
    route_base = "/"

    def index(self):
        self.meta_(title="Hello Pylot!")
        return self.render()

# Example
class Example(Pylot):
    def index(self):
        self.meta_(title="Example Page")
        self.flash_error_("This is an error message set by flash_error_ and called with show_flashed_message()")
        self.flash_success_("This is a success message set by flash_error_ and called with show_flashed_message()")
        return self.render()





# ------------------------------------------------------------------------------
# ENVIRONMENT FOR CONFIG
# Pylot uses configuration class by default, ie: Production(), Development(), (Staging()).
# To switch from one config to another based on the environment we are on,
#
# The methods below will check the type of environment.
#
# The rules below are not a requirement, but a suggestion. You can place your
# files wherever you like.
#
# Create a file on the production server called /.prod_env, to tell the system
# it is on PRODUCTION. /.stage_env for STAGING.
# By default it will fallback to Dev
#
# ------------------------------------------------------------------------------

import os
PROD_ENV = True if os.path.isfile("/.prod_env") else False
STAGE_ENV = True if os.path.isfile("/.stage_env") else False

def is_prod_env():
    return PROD_ENV

def is_stage_env():
    return STAGE_ENV

def is_dev_env():
    return True if not is_prod_env() and not is_stage_env() else False

def get_env():
    """
    Return the string value of the environment: Prod | Stage | Dev
    """
    # Staging server takes priority over prod for security purposes
    if is_prod_env() and not is_stage_env():
        return "Production"
    elif is_stage_env():
        return "Staging"
    else:
        return "Development"

def get_config(config=None):
    """
    Return config class
    :param config : Object - The configuration module containing the eviroment object
    """
    if not config:
        import config
    return getattr(config, get_env())

# ------------------------------------------------------------------------------




"""
Pylot

Pylot is a Flask based framework to help quickly develop web applications, by
adding structure to your views and templates.

Philosophy:

To create a framework that runs everywhere, regardless of the platform, by
providing cloud balh...

It made the following decisions for you: (of course you can change them)


It comes with pre-built components:


And it is still Flask.



Pylot is a framework based on Flask extension that adds structure to both your views and
templates, by mapping them to each other to provide a rapid application development framework.
The extension also comes with Flask-Classy, Flask-Assets, Flask-Mail,
JQuery 2.x, Bootstrap 3.x, Font-Awesome, Bootswatch templates.
The extension also provides pre-made templates for error pages and macros.

http://pylot.io
https://github.com/mardix/pylot

"""

from setuptools import setup, find_packages


def read_pkginfo(filename):
    """
    Help us read the pkginfo without accessing __init__
    """
    COMMENT_CHAR = '#'
    OPTION_CHAR =  '='
    options = {}
    f = open(filename)
    for line in f:
        if COMMENT_CHAR in line:
            line, comment = line.split(COMMENT_CHAR, 1)
        if OPTION_CHAR in line:
            option, value = line.split(OPTION_CHAR, 1)
            option = option.strip()
            value = value.strip()
            options[option] = value
    f.close()
    return options

pkginfo = read_pkginfo('./pylot/pkginfo.py')

__NAME__ = "Pylot"
__version__ = "0.0.4"
__author__ = pkginfo["AUTHOR"]
__license__ = pkginfo["LICENSE"]
__copyright__ = pkginfo["COPYRIGHT"]

setup(
    name=__NAME__,
    version=__version__,
    license=__license__,
    author=__author__,
    author_email='mardix@github.com',
    description="Pylot is a Flask extension that adds structure and map your views and templates together for rapid application development",
    long_description=__doc__,
    url='https://github.com/mardix/pylot/',
    download_url='http://github.com/mardix/pylot/tarball/master',
    py_modules=['pylot'],
    entry_points=dict(console_scripts=[
        'pylot=pylot.cmd:main'
    ]),
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'Flask==0.10.1',
        'Flask-Classy==0.6.10',
        'Flask-Assets==0.10',
        'flask-recaptcha==0.3',
        'flask-login==0.2.11',
        'flask-kvsession==0.6.1',
        'flask-s3==0.1.7',
        'flask-mail==0.9.0',
        'flask-cache==0.13.1',
        'flask-cloudstorage==0.3.1',
        'flask-seasurf==0.2.0',

        'six==1.9.0',
        'Active-SQLAlchemy==0.3.4',
        'passlib==1.6.2',
        'bcrypt==1.1.1',
        'python-slugify==0.1.0',
        'humanize==0.5.1',
        'redis==2.9.1',
        'ses-mailer==0.12.1',
        'mistune==0.5.1'
    ],

    keywords=['flask',
              'pylot',
              'templates',
              'views',
              'classy',
              'pilot',
              'framework',
              "mvc",
              "blueprint"],
    platforms='any',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    zip_safe=False
)



import os
from six.moves.urllib.parse import urlparse, urlunparse, urljoin

url = "memcache://db-01.xnode.io:3306"
up = urlparse(url)
print up
print up.scheme
print up.port
print up.hostname
exit()
import inspect
import wrapt
from flask_classy import FlaskView
def inject(**kwargs):
    pass

def route(rule, **options):
    """A decorator that is used to define custom routes for methods in
    FlaskView subclasses. The format is exactly the same as Flask's
    `@app.route` decorator.
    """

    def decorator(f):
        print f.__name__
        # Put the rule cache on the method itself instead of globally
        if not hasattr(f, '_rule_cache') or f._rule_cache is None:
            f._rule_cache = {f.__name__: [(rule, options)]}
        elif not f.__name__ in f._rule_cache:
            f._rule_cache[f.__name__] = [(rule, options)]
        else:
            f._rule_cache[f.__name__].append((rule, options))

        return f

class P(FlaskView):
    @route("hello")
    def k(self):
        return "Hi"

p = P()
print p.k()

exit()

class Pylot(object):

    @classmethod
    def extends_(cls, kls):
        if inspect.isclass(kls):
            for _name, _val in kls.__dict__.items():
                if not _name.startswith("__"):
                    setattr(cls, _name, _val)
        elif inspect.isfunction(kls):
            setattr(cls, kls.__name__, kls)
        return cls


class Q(Pylot):
    def nice(self):
        return "NICE"

class P(Pylot):
    def set_name(self, name):
        self.name = name

p = P()
q = Q()
#print p.__dict__


@p.extends_
def yo(self, name):
    self.set_name(name)
    print "I'm in self with %s in YO()  " % self.name

@route("hello-world")
@p.extends_
class A(object):
    phone = "123-2453-4343"
    CONST = 54

    @classmethod
    def c2(cls):
        return None

    def hello(self):
        print "This is hello"

    def index(self, name):
        self.set_name(name)
        print "I'm in self with %s " % self.name
        self.hello()

#print dir(p)
#print p.__dict__
p.hello()
p.yo("Koe")
p.index("Lola")
p.index("Nice One")

q.yo("Jones")
q.hello()
print p.phone
print p.CONST

#print f(name="Jones")
exit()


import functools

def deco(model, **kwargs):
    def wrapper(f):
        print f.__name__
        print model
        print kwargs
        return f
    return wrapper


@deco("my model", name="Jbeats")
class Global(object):
    pass


Global()
exit()




s1 = set([])
s2 = set(["B", "C", "W"])

deleted_s = s1 - s2
new_s = s2 - s1

print deleted_s
print new_s
temp3 = [x for x in s1 if x not in s2]

print temp3
exit()
from pylot import utils

class Struct:
    def __init__(self, **entries): self.__dict__.update(entries)

a = Struct(Name="Macxis", Location="Charlotte")

print a.Name
a.Name = "Jose"
print a.Name
print a.Location

print type(a)
def test_mailer_ses():
    pass

def Mailer():
    pass


mailer = Pylot.bind_(Mailer.init_app())()


"""
Contact Page
"""

from pylot import (Pylot,
                   utils,
                   route,
                   abort,
                   redirect,
                   request,
                   url_for)

from pylot.component import (mailer,
                             recaptcha)




def contact_page(**kwargs):

    template_dir = kwargs["template_dir"] if "template_dir" \
                                             in kwargs else "Pylot/ContactPage"
    template_page = template_dir + "/%s.html"

    def wrapper(view):
        @view.extends__
        @route("contact", methods=["GET", "POST"], endpoint="ContactPage")
        def contact(self):
            if request.method == "POST":
                error_message = None
                email = request.form.get("email")
                subject = request.form.get("subject")
                message = request.form.get("message")
                name = request.form.get("name")

                contact_email = self.get_config__("CONTACT_PAGE_EMAIL_RECIPIENT")
                if recaptcha.verify():
                    if not email or not subject or not message:
                        error_message = "All fields are required"
                    elif not utils.is_valid_email(email):
                        error_message = "Invalid email address"
                    if error_message:
                        self.flash_error__(error_message)
                    else:
                        mailer.send_template("contact-us.txt",
                                             to=contact_email,
                                             reply_to=email,
                                             mail_from=email,
                                             mail_subject=subject,
                                             mail_message=message,
                                             mail_name=name
                                            )
                        self.flash_success__("Message sent successfully! We'll get in touch with you soon.")
                else:
                    self.flash_error__("Invalid security code")
                return redirect(url_for("ContactPage"))
            else:
                self.set_meta__(title="Contact Us")
                return self.render(view_template=template_page % "contact")
        return view
    return wrapper